function K = buildkerneldev(P1,P2,lambda);

% K = cspdk(P1,P2,lambda)
% This function calculates the kernel matrix for one a one dimensional
% cubic spline warp with derivative samples available for regression.
% K = cspdk(P1,P2,lambda) returns the kernel matrix for points sets P1
% and P2 useful for interpolating for values of a function at P2.
% K = csp(P1,[],lamdda) returns the kernel matrix with P2=P1 and the
% associated consistency conditions.
% 
% P1 : column vector of points
% P2 : column vector of points
% lambda (optional) : value of the regularization parameter controlling
% the tradeoff between the data term and the smoothing term in the
% regression.

  
  if nargin < 3;
    lambda=0;
  end
  
  if isempty(P2);
    Target=P1;
  else;
    Target=P2;
  end
  
  
  
  [N1,N2]=size(P1);
  [M1,M2]=size(Target);
  
  Xsource=P1(:,1);
  Ysource=P1(:,2);
  
  Xtarget=Target(:,1);
  Ytarget=Target(:,2);
  
  Xsourcemat=repmat(Xsource,1,M1);
  Ysourcemat=repmat(Ysource,1,M1);
  
  Xtargetmat=repmat(Xtarget,1,N1);
  Ytargetmat=repmat(Ytarget,1,N1);
  
  Xd=Xtargetmat-Xsourcemat';
  Yd=Ytargetmat-Ysourcemat';
  
  Xd2=Xd.^2;
  Yd2=Yd.^2;
  
  r2=Xd2+Yd2;
  
  logr=0.5*log(r2+eps);
    
  B=(4*logr+1);
  
  F=(r2.*r2).*logr;
  Fx=Xd.*r2.*B;
  Fy=Yd.*r2.*B;
  
  Fxx=(r2+2*Xd2).*B+4*Xd2;
  Fyy=(r2+2*Yd2).*B+4*Yd2;
  Fxy=2*Xd.*Yd.*(3+4*logr);
  
  C = [ Xtarget.*Xtarget Ytarget.*Ytarget Xtarget.*Ytarget Xtarget Ytarget ...
	ones(M1,1)];
  Cx= [2*Xtarget zeros(M1,1) Ytarget ones(M1,1) zeros(M1,1) zeros(M1,1)];
  Cy= [zeros(M1,1) 2*Ytarget Xtarget zeros(M1,1) ones(M1,1) zeros(M1,1)];
  
    
  A=[F Fx Fy;
     Fx Fxx Fxy;
     Fy Fxy Fyy];
  
  Q=[C;
     Cx;
     Cy];
     
  if isempty(P2);
    K = [A+lambda*eye(3*N1) Q;
	 Q' zeros(6)];
  else;
    K = [A Q];
  end
  